"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _path = require("path");

var _fs = require("fs");

var _parser = require("@babel/parser");

var _traverse = _interopRequireDefault(require("@babel/traverse"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var DIR_NAME = 'plugin-model';
var defaultContent = "import React from 'react';\n\nexport default () => {\n  console.error('[@umijs/plugin-initial-state]: \u68C0\u6D4B\u5230 init \u63D2\u4EF6\u5DF2\u7ECF\u5F00\u542F\uFF0C\u4F46\u662F\u672A\u5728 app.ts/js \u4E2D\u5B9A\u4E49 getInitialInfo \u65B9\u6CD5');\n  return {};\n}\n";

var _default = function _default(api) {
  var paths = api.paths,
      findJS = api.findJS; // 注册 getInitialInfo 方法

  api.addRuntimePluginKey('getInitialInfo');
  var entryFile = findJS((0, _path.join)(paths.absSrcPath, 'app'));

  if (entryFile) {
    api.addPageWatcher([entryFile]);
  }

  api.onGenerateFiles(function () {
    // 读取 app.ts/js 文件内容
    var runtimeInit = '';

    try {
      runtimeInit = (0, _fs.readFileSync)(entryFile).toString();
    } catch (e) {
      runtimeInit = '';
    }

    try {
      var file = (0, _parser.parse)(runtimeInit, {
        sourceType: "module",
        plugins: ["jsx", "typescript"]
      });
      var initStart = 0;
      var initEnd = 0; // 复制 getInitialInfo 到 model

      (0, _traverse.default)(file, {
        enter: function enter(path) {
          if (path.isIdentifier({
            name: "getInitialInfo"
          })) {
            initStart = path.parentPath.parent.start;
            initEnd = path.parentPath.parent.end;
            path.stop();
          }
        }
      });

      if (initEnd) {
        runtimeInit = runtimeInit.slice(initStart, initEnd);
      }

      if (runtimeInit && initEnd) {
        // 写 initialModel
        api.writeTmpFile("".concat(DIR_NAME, "/init.ts"), "import React, { useState, useEffect } from 'react';\nimport { Models } from 'umi';\n\nexport type InitInfo = Models<'@@init'>;\n".concat(runtimeInit, "\n\ntype ThenArg<T> = T extends Promise<infer U> ? U : T\n\nexport default () => {\n  const [loading, setLoading] = useState(false);\n  const [data, setData] = useState<ThenArg<ReturnType<typeof getInitialInfo>> | undefined>(undefined);\n\n  const refresh = async() => {\n    setLoading(true);\n    const asyncFunc = () => new Promise<ReturnType<typeof getInitialInfo>>(res => res(getInitialInfo()));\n    const ret = await asyncFunc();\n    setData(ret);\n    setLoading(false);\n  };\n\n  useEffect(()=>{\n    refresh();\n  }, [])\n\n  return {\n    info: data,\n    refresh,\n    loading,\n  }\n}\n"));
      } else {
        api.writeTmpFile("".concat(DIR_NAME, "/init.ts"), defaultContent);
      }
    } catch (e) {
      api.log.error('[plugin-initial-state]: 入口文件解析失败', e);
    }

    ;
  });
  api.writeTmpFile("".concat(DIR_NAME, "/init.ts"), defaultContent);
  api.addUmiExports([{
    specifiers: ['InitInfo'],
    source: (0, _path.join)(api.paths.absTmpDirPath, DIR_NAME, 'init')
  }]);
  api.register('addExtraModels', function () {
    return [{
      absPath: (0, _path.join)(paths.absTmpDirPath, "".concat(DIR_NAME, "/init.ts")),
      namespace: '@@initialState'
    }];
  });
};

exports.default = _default;